import 'package:flutter/material.dart';
import 'package:flutter/rendering.dart';
import 'package:vibration/vibration.dart';

List<String> monthList = [
  "JAN",
  "FEB",
  "MAR",
  "APR",
  "MAY",
  "JUN",
  "JUL",
  "AUG",
  "SEP",
  "OCT",
  "NOV",
  "DEC"
];

String getDay(int day, int month) {
  DateTime date = DateTime(2020, month, day);
  if (date.weekday == 1) return 'MON';
  if (date.weekday == 2) return 'TUE';
  if (date.weekday == 3) return 'WED';
  if (date.weekday == 4) return 'THU';
  if (date.weekday == 5) return 'FRI';
  if (date.weekday == 6) return 'SAT';
  return 'SUN';
}

int getDayCount(int month) {
  if (month == DateTime.now().month)
    return DateTime.now().day;
  else
    return 1;
}

class SDatePicker extends StatefulWidget {
  final void Function(DateTime) onSelectionChanged;
  final Color mainColor;
  final Color secondColor;
  final Axis scrollDirection;
  final Axis cardDirection;
  final double width;
  final double height;
  final double diameterRatio;
  final double padding;
  final Duration duration;
  final double cardWidth;
  final double cardHeight;

  SDatePicker({
    @required this.onSelectionChanged,
    this.mainColor = Colors.black,
    this.secondColor = Colors.white,
    this.scrollDirection = Axis.vertical,
    this.cardDirection = Axis.horizontal,
    this.width = 100,
    this.height = 100,
    this.diameterRatio = 4,
    this.padding = 4,
    this.duration,
    this.cardWidth = 60,
    this.cardHeight = 25,
  });

  @override
  _SDatePickerState createState() => _SDatePickerState();
}

class _SDatePickerState extends State<SDatePicker> {
  ScrollController controller = FixedExtentScrollController();
  @override
  void initState() {
    setState(() {
      controller = FixedExtentScrollController();
    });

    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    return RotatedBox(
      quarterTurns: widget.scrollDirection == Axis.vertical ? 0 : 1,
      child: Row(
        mainAxisAlignment: MainAxisAlignment.center,
        crossAxisAlignment: CrossAxisAlignment.center,
        children: <Widget>[
          SizedBox(
            height: widget.scrollDirection == Axis.horizontal
                ? MediaQuery.of(context).size.width
                : widget.width,
            width: widget.scrollDirection == Axis.vertical
                ? MediaQuery.of(context).size.height
                : widget.height,
            child: Stack(
              alignment: Alignment.center,
              children: <Widget>[
                RotatedBox(
                  quarterTurns: widget.scrollDirection == Axis.vertical ? 0 : 2,
                  child: ListWheelScrollView(
                    onSelectedItemChanged: (value) {
                      Future.delayed(const Duration(milliseconds: 100), () {
                        Vibration.vibrate(duration: 25, amplitude: 180);
                      });
                      setState(() {
                        selectedM = DateTime.now().month + value;
                        selectedM == DateTime.now().month
                            ? selectedD = DateTime.now().day
                            : selectedD = 1;
                        currentSelection = DateTime(
                            currentSelection.year, selectedM, selectedD);
                        controller.animateTo(0,
                            duration: Duration(milliseconds: 500),
                            curve: Curves.ease);
                      });

                      widget.onSelectionChanged(currentSelection);
                      print(selectedM);
                    },
                    controller: FixedExtentScrollController(),
                    itemExtent: getItemExtent(
                        widget.scrollDirection, widget.cardDirection, 60),
                    diameterRatio: widget.diameterRatio,
                    physics: FixedExtentScrollPhysics(),
                    children: <Widget>[
                      for (int i = DateTime.now().month - 1;
                          i < monthList.length;
                          i++)
                        RotatedBox(
                          quarterTurns:
                              widget.scrollDirection == Axis.vertical ? 0 : 1,
                          child: Row(
                            children: <Widget>[
                              AnimatedContainer(
                                duration: widget.duration != null
                                    ? widget.duration
                                    : Duration(milliseconds: 150),
                                padding: EdgeInsets.all(widget.padding),
                                height: widget.cardDirection == Axis.horizontal
                                    ? widget.cardHeight
                                    : widget.cardWidth,
                                width: widget.cardDirection == Axis.horizontal
                                    ? widget.cardWidth
                                    : widget.cardHeight,
                                decoration: BoxDecoration(
                                    color: i + 1 == selectedM
                                        ? widget.secondColor
                                        : widget.mainColor,
                                    borderRadius: BorderRadius.circular(30)),
                                child: widget.cardDirection == Axis.horizontal
                                    ? Row(
                                        mainAxisAlignment:
                                            MainAxisAlignment.center,
                                        children: <Widget>[
                                          Text(
                                            monthList[i][0],
                                            style: TextStyle(
                                                color: i + 1 == selectedM
                                                    ? widget.mainColor
                                                    : widget.secondColor),
                                          ),
                                          Text(
                                            monthList[i][1],
                                            style: TextStyle(
                                                color: i + 1 == selectedM
                                                    ? widget.mainColor
                                                    : widget.secondColor),
                                          ),
                                          Text(
                                            monthList[i][2],
                                            style: TextStyle(
                                                color: i + 1 == selectedM
                                                    ? widget.mainColor
                                                    : widget.secondColor),
                                          ),
                                        ],
                                      )
                                    : Column(
                                        mainAxisAlignment:
                                            MainAxisAlignment.center,
                                        children: <Widget>[
                                          Text(
                                            monthList[i][0],
                                            style: TextStyle(
                                                color: i + 1 == selectedM
                                                    ? widget.mainColor
                                                    : widget.secondColor),
                                          ),
                                          Text(
                                            monthList[i][1],
                                            style: TextStyle(
                                                color: i + 1 == selectedM
                                                    ? widget.mainColor
                                                    : widget.secondColor),
                                          ),
                                          Text(
                                            monthList[i][2],
                                            style: TextStyle(
                                                color: i + 1 == selectedM
                                                    ? widget.mainColor
                                                    : widget.secondColor),
                                          ),
                                        ],
                                      ),
                              ),
                            ],
                          ),
                        ),
                    ],
                  ),
                ),
              ],
            ),
          ),
          SizedBox(
            height: widget.scrollDirection == Axis.horizontal
                ? MediaQuery.of(context).size.width
                : widget.width,
            width: widget.scrollDirection == Axis.vertical
                ? MediaQuery.of(context).size.height
                : widget.height,
            child: Stack(
              alignment: Alignment.center,
              children: <Widget>[
                RotatedBox(
                  quarterTurns: widget.scrollDirection == Axis.vertical ? 0 : 2,
                  child: ListWheelScrollView(
                    onSelectedItemChanged: (value) {
                      Future.delayed(const Duration(milliseconds: 100), () {
                        Vibration.vibrate(duration: 25, amplitude: 180);
                      });
                      if (selectedM == DateTime.now().month)
                        setState(() {
                          selectedD = DateTime.now().day + value;
                          currentSelection = DateTime(currentSelection.year,
                              currentSelection.month, selectedD);
                        });
                      else
                        setState(() {
                          selectedD = value + 1;
                          currentSelection = DateTime(currentSelection.year,
                              currentSelection.month, selectedD);
                        });
                      widget.onSelectionChanged(currentSelection);
                      print(value);
                    },
                    controller: controller,
                    itemExtent: getItemExtent(
                        widget.scrollDirection, widget.cardDirection, 90),
                    diameterRatio: widget.diameterRatio,
                    physics: FixedExtentScrollPhysics(),
                    children: <Widget>[
                      for (int i = getDayCount(selectedM);
                          i <= daysInMonth(2020, selectedM);
                          i++)
                        RotatedBox(
                          quarterTurns:
                              widget.scrollDirection == Axis.vertical ? 0 : 1,
                          child: Row(
                            children: <Widget>[
                              AnimatedContainer(
                                duration: widget.duration != null
                                    ? widget.duration
                                    : Duration(milliseconds: 150),
                                padding: EdgeInsets.all(widget.padding),
                                height: widget.cardDirection == Axis.horizontal
                                    ? widget.cardHeight
                                    : widget.cardWidth + 25,
                                width: widget.cardDirection == Axis.horizontal
                                    ? widget.cardWidth + 25
                                    : widget.cardHeight,
                                decoration: BoxDecoration(
                                  color: i == selectedD
                                      ? widget.secondColor
                                      : widget.mainColor,
                                  borderRadius: BorderRadius.circular(30),
                                ),
                                child: widget.cardDirection == Axis.horizontal
                                    ? Row(
                                        mainAxisAlignment:
                                            MainAxisAlignment.center,
                                        children: <Widget>[
                                          Text(
                                            i.toString(),
                                            style: TextStyle(
                                                color: i == selectedD
                                                    ? widget.mainColor
                                                    : widget.secondColor),
                                          ),
                                          SizedBox(
                                            height: 10,
                                            width: 10,
                                          ),
                                          Text(
                                            getDay(i, selectedM)[0],
                                            style: TextStyle(
                                                color: i == selectedD
                                                    ? widget.mainColor
                                                    : widget.secondColor),
                                          ),
                                          Text(
                                            getDay(i, selectedM)[1],
                                            style: TextStyle(
                                                color: i == selectedD
                                                    ? widget.mainColor
                                                    : widget.secondColor),
                                          ),
                                          Text(
                                            getDay(i, selectedM)[2],
                                            style: TextStyle(
                                                color: i == selectedD
                                                    ? widget.mainColor
                                                    : widget.secondColor),
                                          ),
                                        ],
                                      )
                                    : Column(
                                        mainAxisAlignment:
                                            MainAxisAlignment.center,
                                        children: <Widget>[
                                          Text(
                                            i.toString(),
                                            style: TextStyle(
                                                color: i == selectedD
                                                    ? widget.mainColor
                                                    : widget.secondColor),
                                          ),
                                          SizedBox(
                                            height: 10,
                                            width: 10,
                                          ),
                                          Text(
                                            getDay(i, selectedM)[0],
                                            style: TextStyle(
                                                color: i == selectedD
                                                    ? widget.mainColor
                                                    : widget.secondColor),
                                          ),
                                          Text(
                                            getDay(i, selectedM)[1],
                                            style: TextStyle(
                                                color: i == selectedD
                                                    ? widget.mainColor
                                                    : widget.secondColor),
                                          ),
                                          Text(
                                            getDay(i, selectedM)[2],
                                            style: TextStyle(
                                                color: i == selectedD
                                                    ? widget.mainColor
                                                    : widget.secondColor),
                                          ),
                                        ],
                                      ),
                              ),
                            ],
                          ),
                        ),
                    ],
                  ),
                ),
              ],
            ),
          ),
        ],
      ),
    );
  }
}

double getItemExtent(Axis scroll, Axis card, int max) {
  if (scroll == Axis.vertical && card == Axis.horizontal) return 30;
  if (scroll == Axis.vertical && card == Axis.vertical) return max.toDouble();
  if (scroll == Axis.horizontal && card == Axis.vertical) return 30;
  return max.toDouble();
}

int selectedM = DateTime.now().month;
int selectedD = DateTime.now().day;

DateTime currentSelection = DateTime.now();

const _daysInMonth = [0, 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31];

int daysInMonth(int year, int month) =>
    (month == DateTime.february && isLeapYear(year)) ? 29 : _daysInMonth[month];

bool isLeapYear(int year) =>
    (year % 4 == 0) && ((year % 100 != 0) || (year % 400 == 0));
